<?php

namespace Plugin\PunchCommerce\src;

use JTL\Cart\Cart;
use JTL\Cart\CartItem;
use JTL\Catalog\Product\Artikel;
use JTL\Plugin\Data\Config;
use JTL\Session\Frontend;
use PunchCommerce\Structs\Basket;
use PunchCommerce\Structs\LineItem;
use PunchCommerce\Structs\Product;

class TransformBasketForPunchCommerce
{
    private Cart $cart;

    public function __construct()
    {
        $this->cart = Frontend::getCart();
    }

    public function __invoke(Config $config): Basket
    {
        $currency = Frontend::getCurrency();

        $basket = new Basket();
        foreach ($this->cart->PositionenArr as $position) {
            $product = new Product();

            $product->id = $position->Artikel->kArtikel;
            $product->brand = $position->Artikel->cHersteller ?? 'NOT SET';
            $product->brandOrdernumber = $position->Artikel->cHAN ?? 'NOT SET';
            $product->category = $position->Artikel->gibKategorie();
            $product->ordernumber = $position->cArtNr;
            $product->title = $position->cName['ger'];
            $product->description = $position->Artikel->cBeschreibung;
            $product->descriptionLong = $position->Artikel->cBeschreibung;
            $product->price = $position->fPreisEinzelNetto;
            $product->taxRate = (float)$position->Artikel->fMwSt;
            $product->packagingUnit = empty($position->cEinheit) ? 'Stück' : $position->cEinheit;
            $product->weight = $position->fGesamtgewicht;
            $product->currency = $currency->getCode();
            $product->shippingTime = $position->Artikel->fLieferzeit;
            $product->classificationType = 'ECLASS-11.1';
            $product->classification = '13000000';
            $product->active = true;

            $product = $this->mapCustomFields($product, $position, $config);
            $lineItem = new LineItem($product, $position->nAnzahl);
            $basket->add($lineItem);
        }

        return $basket;
    }

    private function mapCustomFields(Product $product, CartItem $position, Config $config): Product
    {
        for ($i = 1; $i < 11; $i++) {
            $target = "customField{$i}";
            $source = $config->getValue("customfield_{$i}");

            if ($source !== null) {
                if (str_contains($source, '.')) {
                    // Array of potential attributes of class "Artikel"
                    $source = explode('.', $source);

                    /** @var Artikel $value */
                    $value = $position->Artikel;
                    foreach ($source as $item) {
                        if (is_array($value)) {
                            if (! array_key_exists($item, $value)) {
                                continue;
                            }

                            $value = $value[$item];
                        } else {
                            $value = $value?->$item;
                        }
                    }

                    if (! is_array($value) && !empty($value)) {
                        $product->$target = $value;
                    }

                } else if ($position->Artikel !== null && property_exists($position->Artikel, $source)) {
                    $product->$target = $position->Artikel->$source;
                }
            }
        }

        return $product;
    }
}
